#include "variav.h"
#include "jogo.h"
#include "output.h"

/* -----------
 * new_piece()
 * coloca a nova peca no tabuleiro
 * ---------- */
void new_piece()
{
    int i,j;

    search_full_line();

    /* cria new_board */
    for(i=1;i<=COLS;i++)
        for(j=1;j<=LINES;j++)
            new_board[i][j] = board[i][j];

    piece = next;
    next  = random() % 7;

    /* show_next_piece(next); */

    /* copia o desenho da peca para current_draw */
    for(i=0;i<=3;i++)
        for(j=0;j<=3;j++)
            current_draw[i][j] = draw[piece][i][j];

    pos_x = 4;
    pos_y = 1;
    old_pos_x = 0;
    old_pos_y = 0;

    /* testa fim do jogo */
    if(is_move_ok(pos_x,pos_y)==0) game_over();

    print_places(0);
}

/* --------------
 * register_move()
 * registra o movimento da peca no tabuleiro
 * -------------- */
void register_move(int turned)
{
    register int i,j;

    /* apaga a velha peca */
    if(turned==0) {
        for(i=0;i<=3;i++)
            for(j=0;j<=3;j++)
                if(board[i+old_pos_x][j+old_pos_y]>0 && current_draw[j][i]>0)
                    board[i+old_pos_x][j+old_pos_y] = 0;
    } else if (turned==1) {
        for(i=0;i<=3;i++)
            for(j=0;j<=3;j++)
                if(board[i+old_pos_x][j+old_pos_y]>0 && old_draw[j][i]>0)
                    board[i+old_pos_x][j+old_pos_y] = 0;
    }

    /* coloca a nova peca */
    for(i=0;i<=3;i++)
        for(j=0;j<=3;j++)
            if((current_draw[j][i]>0)&&(i+pos_x<=COLS)&&(j+pos_y<=LINES))
                board[i+pos_x][j+pos_y] = current_draw[j][i];

    print_places(turned);
}

/*------------
 * is_move_ok(peca,destino_x,destino_y)
 * checa se o movimento pode ser feito
 *------------ */
int is_move_ok(int x, int y)
{
    register int i,j;
    int ok = 1;

    /* checa se a peca vai sobrepor alguma casa ocupada */
    for(i=0;i<=3;i++)
        for(j=0;j<=3;j++)
            if(!((i+x>COLS)||(j+y>LINES)))
                if((new_board[i+x][j+y]>0)&&(current_draw[j][i]>0))
                    ok = 0;

    /* checa se a peca cai fora do tabuleiro */
    for(i=0;i<=3;i++) {
        for(j=0;j<=3;j++) {
            if((current_draw[j][i]>0)&&(j+y>LINES))
                ok = 0;
            if((current_draw[j][i]>0)&&(i+x>COLS))
                ok = 0;
            if((current_draw[j][i]>0)&&(i+x<1))
                ok = 0;
        }
    }

    return ok;
}

/*------------
 *turn_piece()
 *vira a peca
 *------------ */
void turn_piece()
{
    register int i,j;
    int temp[4][4];

    for(i=0;i<=3;i++) {
        for(j=0;j<=3;j++) {
            old_draw[j][i] = current_draw[j][i];
            temp[i][j] = current_draw[i][j];
        }
    }

    if((piece!=0)&&(piece!=4)) {
        for(i=0;i<=2;i++)
            for(j=0;j<=2;j++)
                current_draw[i][j] = temp[inv(j)][i];
    } else if(piece==0) {
        for(i=0;i<=3;i++)
            for(j=0;j<=3;j++)
                current_draw[i][j] = temp[j][i];
    }
}

/*------------
 * test_turn_piece()
 * vira a peca
 *------------ */
int test_turn_piece()
{
    register int i,j;
    int temp[4][4];
    int ok=1;

    /* Inicializa temp[][] */
    for(i=0;i<=3;i++)
        for(j=0;j<=3;j++)
	   temp[j][i]=0;

    if((piece!=0)&&(piece!=4)) {
        for(i=0;i<=2;i++)
            for(j=0;j<=2;j++)
                temp[i][j] = current_draw[inv(j)][i];
    } else if(piece==0) {
        for(i=0;i<=3;i++)
            for(j=0;j<=3;j++)
                temp[i][j] = current_draw[j][i];
    }

    /* checa se a peca vai sobrepor alguma casa ocupada */
    for(i=0;i<=3;i++)
        for(j=0;j<=3;j++)
            if(!((i+pos_x>COLS)||(j+pos_y>LINES)))
                if((new_board[i+pos_x][j+pos_y]>0)&&(temp[j][i]>0))
                    ok = 0;

    /* checa se parte da peca nao vai cair fora */
    for(i=0;i<=3;i++) {
        for(j=0;j<=3;j++) {
            if(((i+pos_x < 1)||(i+pos_x>COLS)||(j+old_pos_y>LINES)) && temp[j][i]>0)
                ok = 0;
        }
    }
    return ok;
}


/* -----------------
 * -> inv(num)
 * inverte o numero (para turn_piece())
 * ----------------- */
int inv(int num)
{
    int a = num;
    if(a==0) a=2;
    else if(a==2) a=0;
    return a;
}

/* -----------------
 * -> search_full_line()
 * procura linhas completas e as apaga
 * ----------------- */
void search_full_line()
{
    register int i,j,k,linha;
    int qtd = 0;

    for(i=1;i<=COLS;i++)
        for(j=1;j<=LINES;j++)
            old_board[i][j] = board[i][j];

    for(i=1;i<=LINES;i++) {
        linha = 1;
        for(j=1;j<=COLS;j++)
            if(old_board[j][i]==0) linha = 0;
        /* apaga as linhas */
        if(linha==1) {
            for(j=i;j>=2;j--)
                for(k=1;k<=COLS;k++)
                    board[k][j]=board[k][j-1];
            qtd++;
        }
    }
    if(qtd==1) score += 100;
    if(qtd==2) score += 300;
    if(qtd==3) score += 600;
    if(qtd==4) score += 1000;

    print_deleted_lines();
}
